{% load i18n %}
{% load navactive %}

<li>
    <a class="{% navactive request item.navactive|join:" " %}" href="{% url item.url_name %}">
        <i class="{{ item.classes }}"></i> {% trans item.text %}
    </a>
</li>
