{% load static %}
<!-- Start jQuery datetimepicker CSS -->
<link href="https://cdnjs.cloudflare.com/ajax/libs/jquery-datetimepicker/2.3.7/jquery.datetimepicker.min.css" rel="stylesheet" type="text/css">
<!-- End jQuery datetimepicker CSS -->
