{% load static %}
<!-- Start jQuery datetimepicker js -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/jquery-datetimepicker/2.3.7/jquery.datetimepicker.min.js"></script>
<!-- End jQuery datetimepicker js -->
