# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-12 13:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('authentication', '0007_remove_authservicesinfo_is_blue'),
        ('eveonline', '0001_initial'),
        ('auth', '0001_initial'),
    ]

    operations = [
    ]
